#ifndef COMMON_CK
#define COMMON_CK

#include "DispMessage.h"

#include "rnkissM.h"

using namespace System;
using namespace MyRNKISS;
using namespace me_anal_mcmc;

namespace cmmn {

	extern int n;

	extern double * comp, * est;

	extern const double maxStV;
	extern double maxEstV;

	extern const int nPoints;

	extern double varErr;

	extern double param_a, param_b, param_c;

	extern double mean_a, mean_b, mean_c, mean_sgm, map_a, map_b, map_c, map_s;

	extern int ck_a, ck_b, ck_c, ck_s;

	double sqr( double v );
	
	double scale_sgm( double acpt_r );

	double psy0( double phi, double a, double b);

	double psy1( double phi, double a, double b, double c);


	ref class MCMC {
		public:
			int NSimu, N;
			double * a, * b, * c, * sgm, * x, * y;
			double min_x, aSgm, bSgm, cSgm, sgmSgm;
			int accpt_a, accpt_b, accpt_c, accpt_sgm;
			myrn * rn;
			String ^ msg;

			MCMC(): NSimu(10000), accpt_a(0), accpt_b(0), accpt_c(0), accpt_sgm(0) { }

			void Init( int vN, double * vx, double * vy,
						double a0, double vaSgm, double b0, double vbSgm, double c0, double vcSgm, double sgm0, double vsgmSgm ){
				N = vN;
				x = vx;
				y = vy;
				a = new double[NSimu];
				b = new double[NSimu];
				c = new double[NSimu];
				sgm = new double[NSimu];
				a[0] = a0;
				aSgm = vaSgm;
				b[0] = b0;
				bSgm = vbSgm;
				c[0] = c0;
				cSgm = vcSgm;
				sgm[0] = sgm0;
				sgmSgm = vsgmSgm;

				min_x = x[0];
				for (int i = 1; i < N; i++)
					if (min_x > x[i]) min_x = x[i];
			}

			double log_kernel( double a, double b, double c, double sgm ){
				double v = 0.0;
				for (int i = 0; i < N; i++)
					v += sqr( (Math::Log(y[i]) - (Math::Log(a) + b * Math::Log(x[i] - c))) / sgm );
				return -0.5 * v;
			}

			DispMessage ^ cf;

			void do_mcmc(){
				cf = gcnew DispMessage();
				cf->Show();
				cf->ShowMessage("MCMC started.");
				Application::DoEvents();

				accpt_a = accpt_b = accpt_c = accpt_sgm = 0;
				for (int itrial = 0; itrial < NSimu-1; itrial++){
					if (itrial % 1000 == 0){
						cf->ShowMessage(itrial.ToString() + "/" + NSimu.ToString() + "..." + msg);
						Application::DoEvents();
					}

					double a0 = a[itrial], at, b0 = b[itrial], bt, c0 = c[itrial], ct, sgm0 = sgm[itrial], sgmt;
					double y = rn->normalMS( a[0], aSgm );
					if (y > 0.0){
						double v = log_kernel( y, b0, c0, sgm0 ) - log_kernel( a0, b0, c0, sgm0 );
						double a = Math::Exp( v );
						if (rn->uni() < a){
							at = y;
							accpt_a++;
						}else{
							at = a0;
						}
					}else{
							at = a0;
					}
					a[itrial+1] = at;

					y = rn->normalMS( b0, bSgm );
					if (y > 0.0){
						double v = log_kernel( at, y, c0, sgm0 ) - log_kernel( at, b0, c0, sgm0 );
						double a = Math::Exp( v );
						if (rn->uni() < a){
							bt = y;
							accpt_b++;
						}else{
							bt = b0;
						}
					}else{
							bt = b0;
					}
					b[itrial+1] = bt;

					y = rn->normalMS( c0, cSgm );
					if (y < min_x){
						double v = log_kernel( at, bt, y, sgm0 ) - log_kernel( at, bt, c0, sgm0 );
						double a = Math::Exp( v );
						if (rn->uni() < a){
							ct = y;
							accpt_c++;
						}else{
							ct = c0;
						}
					}else{
							ct = c0;
					}
					c[itrial+1] = ct;

					y = rn->normalMS( sgm0, sgmSgm );
					if (y > 0.0){
						double v = -N * Math::Log(y) - (-N * Math::Log(sgm0));
							v += log_kernel( at, bt, ct, y ) - log_kernel( at, bt, ct, sgm0 );
						double a = Math::Exp( v );
						if (rn->uni() < a){
							sgmt = y;
							accpt_sgm++;
						}else{
							sgmt = sgm0;
						}
					}else{
							sgmt = sgm0;
					}
					sgm[itrial+1] = sgmt;

				}

				cf->Close();
			}

	};


	void qSort( double * d, int Lb, int Ub );

	double est_distri( double x, double * d, int N, double sd );

	void calc_CI( double * d, int NSimu, double & CIL, double & Med, double & CIR );


}

#endif
